import 'dart:collection';

class ListUtils {
  /// Returns a list of unique values from the given parameter `items`.
  /// The order of all unique items are maintained.
  ///
  /// If the given parameter is null an empty list will be returned.
  static List<T> removeDuplicates<T>({required List<T>? items}) {
    if (items == null) {
      return [];
    }
    List<T> uniqueValues = List.empty(growable: true);
    for (var item in items) {
      if (!uniqueValues.contains(item)) {
        uniqueValues.add(item);
      }
    }
    return uniqueValues;
  }

  /// Returns a list of unique values identified by the given function parameter `by`.
  /// The order of all unique items are maintained.
  ///
  /// If the given parameter is null an empty list will be returned.
  static List<T> removeDuplicatesBy<T>({
    required List<T>? items,
    required dynamic Function(T a) by,
  }) {
    if (items == null) {
      return [];
    }
    Map<dynamic, T> uniqueValues = HashMap<dynamic, T>();
    for (var item in items) {
      if (!uniqueValues.containsKey(by(item))) {
        uniqueValues[by(item)] = item;
      }
    }
    return uniqueValues.values.toList();
  }

  /// Returns an empty list if the given parameter `items` is null.
  /// Otherwise the given list is returned.
  static List<T> emptyIfNull<T>({required List<T>? items}) {
    return items == null ? [] : items;
  }

  /// Returns true if the given list `items` contains at least one duplicated value.
  /// Otherwise false.
  ///
  /// If the given parameter is null false will be returned.
  static bool containsDuplicates<T>({required List<T>? items}) {
    if (items == null) {
      return false;
    }
    List<T> uniqueValues = [];
    for (var item in items) {
      if (uniqueValues.contains(item)) {
        return true;
      } else {
        uniqueValues.add(item);
      }
    }
    return false;
  }

  /// Sorts the given list by the given parameter `by`.
  /// Default sorting order is ascending. This can be changed by setting the `descending` to true.
  static void sortByNumber<T>({
    required List<T> items,
    required num Function(T a) by,
    bool descending = false,
  }) {
    for (int i = 0; i < items.length; i++) {
      for (int j = i; j < items.length; j++) {
        if (descending) {
          if (by(items[i]) < by(items[j])) {
            dynamic currentValue = items[j];
            items[j] = items[i];
            items[i] = currentValue;
          }
        } else {
          if (by(items[i]) > by(items[j])) {
            dynamic currentValue = items[j];
            items[j] = items[i];
            items[i] = currentValue;
          }
        }
      }
    }
  }

  /// Sorts the given list by the length of the given parameter `by`.
  /// Default sorting order is ascending. This can be changed by setting the `descending` to true.
  static void sortByString<T>({
    required List<T> items,
    required String Function(T a) by,
    bool descending = false,
  }) {
    for (int i = 0; i < items.length; i++) {
      for (int j = i; j < items.length; j++) {
        if (descending) {
          if (by(items[i]).length < by(items[j]).length) {
            T currentValue = items[j];
            items[j] = items[i];
            items[i] = currentValue;
          }
        } else {
          if (by(items[i]).length > by(items[j]).length) {
            T currentValue = items[j];
            items[j] = items[i];
            items[i] = currentValue;
          }
        }
      }
    }
  }
}
